window.Gleam.initSearch([{"doc":"pears","title":"pears","content":"# Pears - A parser combinator library for Gleam\n\n[![Package Version](https://img.shields.io/hexpm/v/pears)](https://hex.pm/packages/pears)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/pears/)\n\n🚧️ **This library is in early development. Expect breaking changes.** 🚧\n\n## Installation\n\n```sh\ngleam add pears\n```\n\n## Usage\n\n```gleam\nimport pears.{type Parser, Parsed}\nimport pears/chars.{type Char, number}\nimport pears/combinators.{alt, between, just, lazy, map, sep_by0}\n\npub type Tree(a) {\n  Leaf(a)\n  Node(List(Tree(a)))\n}\n\nfn tree_parser(p: Parser(Char, a)) -&gt; Parser(Char, Tree(a)) {\n  let tree = lazy(fn() { tree_parser(p) })\n  let leaf = map(p, Leaf)\n  let node =\n    tree\n    |&gt; sep_by0(just(&quot;,&quot;))\n    |&gt; between(just(&quot;[&quot;), just(&quot;]&quot;))\n    |&gt; map(Node)\n  alt(leaf, node)\n}\n\npub fn main() {\n  let parse_result =\n    &quot;[1,[2,3],4]&quot;\n    |&gt; chars.input()\n    |&gt; tree_parser(number())\n\n  let assert Ok(Parsed([], Node([Leaf(1), Node([Leaf(2), Leaf(3)]), Leaf(4)]))) =\n    parse_result\n}\n```\n\nFurther documentation can be found at [https://hexdocs.pm/pears](https://hexdocs.pm/pears/pears.html).\n\nSee the [test](./test) directory for more examples.\n\n## What&#39;s missing?\n\n- Proper error handling\n- Test helpers\n- ...\n\n## Development\n\n```sh\ngleam test  # Run the tests\ngleam shell # Run an Erlang shell\n```\n","url":"index.html"},{"doc":"pears/input","title":"Input","content":"pub type Input(i) {\n  Input(tokens: List(i), cursor: Int)\n}\n Represents a list of tokens that can be consumed by a parser, along with a cursor indicating the current position.\nInput(tokens: List(i), cursor: Int)\n\n\nSynonyms:\npears/input.Input\npears/input Input","url":"pears/input.html#Input"},{"doc":"pears/input","title":"at_end","content":"pub fn at_end(input: Input(a)) -&gt; Bool\n Returns true if the input has been fully consumed\nSynonyms:\npears/input.at_end\npears/input at_end","url":"pears/input.html#at_end"},{"doc":"pears/input","title":"get","content":"pub fn get(input: Input(a)) -&gt; Option(a)\n\nSynonyms:\npears/input.get\npears/input get","url":"pears/input.html#get"},{"doc":"pears/input","title":"get_n","content":"pub fn get_n(input: Input(a), n: Int) -&gt; List(a)\n\nSynonyms:\npears/input.get_n\npears/input get_n","url":"pears/input.html#get_n"},{"doc":"pears/input","title":"next","content":"pub fn next(input: Input(a)) -&gt; Input(a)\n\nSynonyms:\npears/input.next\npears/input next","url":"pears/input.html#next"},{"doc":"pears/input","title":"next_n","content":"pub fn next_n(input: Input(a), n: Int) -&gt; Input(a)\n\nSynonyms:\npears/input.next_n\npears/input next_n","url":"pears/input.html#next_n"},{"doc":"pears/input","title":"pears/input","content":" This module defines the `Input(i)` type that represents a list of tokens that can be consumed by a parser.\n It also provides some helper functions for working with `Input(i)`.","url":"pears/input.html"},{"doc":"pears","title":"ParseError","content":"pub type ParseError(i) {\n  UnexpectedEndOfInput(input: Input(i), expected: List(String))\n  UnexpectedToken(\n    input: Input(i),\n    expected: List(String),\n    found: i,\n  )\n}\n\nUnexpectedEndOfInput(input: Input(i), expected: List(String))\n\n\nUnexpectedToken(\n  input: Input(i),\n  expected: List(String),\n  found: i,\n)\n\n\nSynonyms:\npears.ParseError\npears ParseError","url":"pears.html#ParseError"},{"doc":"pears","title":"ParseResult","content":"pub type ParseResult(i, a) =\n  Result(Parsed(i, a), ParseError(i))\n The result of a parser. It is either a `Parsed` or a `ParseError`.\n\nSynonyms:\npears.ParseResult\npears ParseResult","url":"pears.html#ParseResult"},{"doc":"pears","title":"Parsed","content":"pub type Parsed(i, a) {\n  Parsed(input: Input(i), value: a)\n}\n Returned by a parser when it is successful. It contains the remaining input and the parsed value.\nParsed(input: Input(i), value: a)\n\n\nSynonyms:\npears.Parsed\npears Parsed","url":"pears.html#Parsed"},{"doc":"pears","title":"Parser","content":"pub type Parser(i, a) =\n  fn(Input(i)) -&gt; ParseResult(i, a)\n The generic parser type. It&#39;s an alias for a function that takes an `Input(i)` and returns a `ParseResult(i, a)`.\n\nSynonyms:\npears.Parser\npears Parser","url":"pears.html#Parser"},{"doc":"pears","title":"ok","content":"pub fn ok(\n  input: Input(a),\n  value: b,\n) -&gt; Result(Parsed(a, b), ParseError(a))\n\nSynonyms:\npears.ok\npears ok","url":"pears.html#ok"},{"doc":"pears","title":"parse","content":"pub fn parse(\n  i: List(a),\n  p: fn(Input(a)) -&gt; Result(Parsed(a, b), ParseError(a)),\n) -&gt; Result(Parsed(a, b), ParseError(a))\n Helper function that takes a parser and an input and runs the parser on the input.\nSynonyms:\npears.parse\npears parse","url":"pears.html#parse"},{"doc":"pears","title":"parse_string","content":"pub fn parse_string(\n  i: String,\n  p: fn(Input(String)) -&gt;\n    Result(Parsed(String, a), ParseError(String)),\n) -&gt; Result(Parsed(String, a), ParseError(String))\n Helper function that takes a string and a parser and runs the parser on the string.\nSynonyms:\npears.parse_string\npears parse_string","url":"pears.html#parse_string"},{"doc":"pears","title":"pears","content":" ## A parser combinator library for gleam\n\n Writing a parser using pears is easy, you just combine functions that\n parse smaller parts of the input into a function that parses the whole input.\n\n Here&#39;s an example of a parser that parses the letter &#39;a&#39;:\n\n ```gleam\n let a_parser = just(&quot;a&quot;)\n ```\n\n We can run this parser using the `parse_string` function:\n\n ```gleam\n parse_string(&quot;abc&quot;, a_parser)\n // =&gt; Ok(Parsed(_, &quot;a&quot;))\n ```\n\n It returns a `Result` that gives us the the parsed value and the remaining input.\n\n Under the hood all `parse_string` does is convert the input string into a list of graphemes and then\n call given parser, which is just a function, passing the input to it.\n\n It is equivalent to the following:\n\n ```gleam\n let input = string.to_graphemes(&quot;abc&quot;)\n a_parser()(input)\n // =&gt; Ok(Parsed(_, &quot;a&quot;))\n ```\n\n ### Combinators\n\n Combining parsers using combinators is how you build more complex parsers.\n A combinator is just a function that one or more parsers and returns a new parser.\n\n Let&#39;s say we want to parse the letter &#39;a&#39; followed by the letter &#39;b&#39;, we can use the `pair` combinator for that:\n\n ```gleam\n let ab_parser = pair(just(&quot;a&quot;), just(&quot;b&quot;))\n parse_string(&quot;abc&quot;, ab_parser)\n // =&gt; Ok(Parsed(_, #(&quot;a&quot;, &quot;b&quot;)))\n ```\n\n The `pair` combinator takes two parsers and returns a new parser that runs the first parser and then the second\n parser, returning both results as a tuple if both are successful.\n\n To create parsers that are actually useful we need to be able to branch based on the input,\n for that we can use the `alt` combinator:\n\n ```gleam\n let a_or_b_parser = alt(just(&quot;a&quot;), just(&quot;b&quot;))\n parse_string(&quot;abc&quot;, a_or_b_parser)\n // =&gt; Ok(Parsed(_, &quot;a&quot;))\n parse_string(&quot;bac&quot;, a_or_b_parser)\n // =&gt; Ok(Parsed(_, &quot;b&quot;))\n ```\n\n In cases where there are more than two options, we can use the `choice` combinator:\n\n ```gleam\n let a_b_or_c_parser = choice(just(&quot;a&quot;), just(&quot;b&quot;), just(&quot;c&quot;))\n parse_string(&quot;abc&quot;, a_b_or_c_parser)\n // =&gt; Ok(Parsed(_, &quot;a&quot;))\n parse_string(&quot;bca&quot;, a_b_or_c_parser)\n // =&gt; Ok(Parsed(_, &quot;b&quot;))\n parse_string(&quot;cab&quot;, a_b_or_c_parser)\n // =&gt; Ok(Parsed(_, &quot;c&quot;))\n ```\n\n The next crucial combinators are `many0` and `many1`.\n They allow us to parse zero or more or one or more repetitions of a parser:\n\n ```gleam\n let abc0_parser = many0(a_b_or_c_parser)\n parse_string(&quot;abc&quot;, abc0_parser)\n // =&gt; Ok(Parsed(_, [&quot;a&quot;, &quot;b&quot;, &quot;c&quot;]))\n parse_string(&quot;cab&quot;, abc0_parser)\n // =&gt; Ok(Parsed(_, [&quot;c&quot;, &quot;a&quot;, &quot;b&quot;]))\n parse_string(&quot;abcbcacab&quot;, abc0_parser)\n // =&gt; Ok(Parsed(_, [&quot;a&quot;, &quot;b&quot;, &quot;c&quot;, &quot;b&quot;, &quot;c&quot;, &quot;a&quot;, &quot;c&quot;, &quot;a&quot;, &quot;b&quot;]))\n ```\n\n Sometimes we want to parse something and then ignore the result,\n for that we can use the `left` and `right` combinators.\n\n In this example we parse the letter &#39;a&#39; followed by the letter &#39;b&#39; and ignore the result of the first parser:\n\n ```gleam\n let a_followed_by_b_parser = left(just(&quot;a&quot;), just(&quot;b&quot;))\n parse_string(&quot;abc&quot;, a_followed_by_b_parser)\n // =&gt; Ok(Parsed(_, &quot;a&quot;))\n ```\n\n The `right` combinator works the same way but ignores the result of the second parser.\n\n ```gleam\n let b_preceeded_by_a_parser = right(just(&quot;a&quot;), just(&quot;b&quot;))\n parse_string(&quot;abc&quot;, b_preceeded_by_a_parser)\n // =&gt; Ok(Parsed(_, &quot;b&quot;))\n ```\n\n These building blocks are enough to build many combinators that can parse complex data structures.\n For example the `sep_by0` and `sep_by1` combinators which can be used to parse lists of items\n separated by a delimiter.\n\n ```\n let comma_separated_letters = sep_by0(a_or_b_or_c_parser, just(&quot;,&quot;))\n parse_string(&quot;a,b,c&quot;, comma_separated_letters)\n // =&gt; Ok(Parsed(_, [&quot;a&quot;, &quot;b&quot;, &quot;c&quot;]))\n ```\n\n ### Transforming the results\n\n The `map` combinator can be used to transform the result of a parser.\n It takes a parser and a function that takes the result of the parser and returns a new value.\n\n Let&#39;s say we want to parse our abc letters and transform them into the following types:\n\n ```gleam\n type Letter {\n   A\n   B\n   C\n   Other(String)\n }\n\n type Letters {\n   Letters(List(Letter))\n }\n ```\n\n First we need to define parsers for each letter:\n\n ```gleam\n let a_parser = map(just(&quot;a&quot;), fn(_letter) { Letter.A })\n let b_parser = map(just(&quot;b&quot;), fn(_letter) { Letter.B })\n let c_parser = map(just(&quot;c&quot;), fn(_letter) { Letter.C })\n ```\n\n In our map function we ignore the input and return the corresponding letter. We will get to the `Other` case later.\n\n Now that we have parsers that take a char and we can combine them as previously and then use the `map` combinator\n to transform the result into `Letters`:\n\n ```gleam\n let letters_parser =\n   choice([a_parser, b_parser, c_parser])\n   |&gt; map(Letters)\n\n parse_string(&quot;abc&quot;, letters_parser)\n // =&gt; Ok(Parsed(_, Letters([Letter.A, Letter.B, Letter.C])))\n ```\n\n Let&#39;s add the `Other` case to our parser, for that we can use the satisfying combinator which takes a function\n that returns a boolean indicating if the input matches or not. We can use it to check if the input is a letter\n of the alphabet.\n\n While we are at it, let&#39;s also use `to` instead of `map` to transform our a, b, and c parsers into the `Letter` type.\n `to` is a shorthand for `map` where you want to ignore the input and just return a new value.\n\n ```gleam\n let a_parser = to(just(&quot;a&quot;), Letter.A)\n let b_parser = to(just(&quot;b&quot;), Letter.B)\n let c_parser = to(just(&quot;c&quot;), Letter.C)\n let other_parser = satisfying(fn(char) { is_alphabetic(char) })\n\n let letters_parser =\n  choice([a_parser, b_parser, c_parser, other_parser])\n  |&gt; map(Letters)\n\n parse_string(&quot;abcd&quot;, letters_parser)\n // =&gt; Ok(Parsed(_, Letters([Letter.A, Letter.B, Letter.C, Letter.Other(&quot;d&quot;)]))\n ```\n\n ### More examples\n\n Please have a look at the tests for more complex examples, such as parsing JSON or Brainf*ck.","url":"pears.html"},{"doc":"pears/combinators","title":"Predicate","content":"pub type Predicate(i) =\n  fn(i) -&gt; Bool\n A predicate function that returns a boolean value for a given input item of type `i`.\n\nSynonyms:\npears/combinators.Predicate\npears/combinators Predicate","url":"pears/combinators.html#Predicate"},{"doc":"pears/combinators","title":"alt","content":"pub fn alt(\n  parser_1: fn(Input(a)) -&gt; Result(Parsed(a, b), ParseError(a)),\n  parser_2: fn(Input(a)) -&gt; Result(Parsed(a, b), ParseError(a)),\n) -&gt; fn(Input(a)) -&gt; Result(Parsed(a, b), ParseError(a))\n Tries to apply the first parser, if it fails it applies the second parser.\nSynonyms:\npears/combinators.alt\npears/combinators alt","url":"pears/combinators.html#alt"},{"doc":"pears/combinators","title":"any","content":"pub fn any() -&gt; fn(Input(a)) -&gt;\n  Result(Parsed(a, a), ParseError(a))\n Consumes any single item from the input except for the end of file.\nSynonyms:\npears/combinators.any\npears/combinators any","url":"pears/combinators.html#any"},{"doc":"pears/combinators","title":"between","content":"pub fn between(\n  parser: fn(Input(a)) -&gt; Result(Parsed(a, b), ParseError(a)),\n  open: fn(Input(a)) -&gt; Result(Parsed(a, c), ParseError(a)),\n  close: fn(Input(a)) -&gt; Result(Parsed(a, d), ParseError(a)),\n) -&gt; fn(Input(a)) -&gt; Result(Parsed(a, b), ParseError(a))\n Applies the given parser between the open and close parsers, returning the result of the innner parser.\nSynonyms:\npears/combinators.between\npears/combinators between","url":"pears/combinators.html#between"},{"doc":"pears/combinators","title":"choice","content":"pub fn choice(\n  parsers: List(\n    fn(Input(a)) -&gt; Result(Parsed(a, b), ParseError(a)),\n  ),\n) -&gt; fn(Input(a)) -&gt; Result(Parsed(a, b), ParseError(a))\n Tries to apply the given parsers in order and returns the result of the first one that succeeds.\nSynonyms:\npears/combinators.choice\npears/combinators choice","url":"pears/combinators.html#choice"},{"doc":"pears/combinators","title":"do_choice","content":"pub fn do_choice(\n  parsers: List(\n    fn(Input(a)) -&gt; Result(Parsed(a, b), ParseError(a)),\n  ),\n  expected: List(String),\n) -&gt; fn(Input(a)) -&gt; Result(Parsed(a, b), ParseError(a))\n\nSynonyms:\npears/combinators.do_choice\npears/combinators do_choice","url":"pears/combinators.html#do_choice"},{"doc":"pears/combinators","title":"eof","content":"pub fn eof() -&gt; fn(Input(a)) -&gt;\n  Result(Parsed(a, Nil), ParseError(a))\n Consumes the end of the input and returns a Nil value. Fails if there is any input left.\nSynonyms:\npears/combinators.eof\npears/combinators eof","url":"pears/combinators.html#eof"},{"doc":"pears/combinators","title":"just","content":"pub fn just(\n  item: a,\n) -&gt; fn(Input(a)) -&gt; Result(Parsed(a, a), ParseError(a))\n Consumes a single item from the input that is equal to the given item.\nSynonyms:\npears/combinators.just\npears/combinators just","url":"pears/combinators.html#just"},{"doc":"pears/combinators","title":"labelled","content":"pub fn labelled(\n  parser: fn(Input(a)) -&gt; Result(Parsed(a, b), ParseError(a)),\n  label: String,\n) -&gt; fn(Input(a)) -&gt; Result(Parsed(a, b), ParseError(a))\n\nSynonyms:\npears/combinators.labelled\npears/combinators labelled","url":"pears/combinators.html#labelled"},{"doc":"pears/combinators","title":"lazy","content":"pub fn lazy(\n  f: fn() -&gt; fn(Input(a)) -&gt; Result(Parsed(a, b), ParseError(a)),\n) -&gt; fn(Input(a)) -&gt; Result(Parsed(a, b), ParseError(a))\n Lazily applies the given parser allowing for recursive parsers.\n\n ## Examples\n\n Parsing a recursive list of chars:\n\n \n pub type Tree(a) {\n   Leaf(a)\n   Node(List(Tree(a)))\n }\n\n fn tree_parser() -&gt; Parser(Char, Tree(Int)) {\n   let tree = lazy(tree_parser)\n   let leaf = map(number(), Leaf)\n   let node =\n     tree\n     |&gt; sep_by0(just(&quot;,&quot;))\n     |&gt; between(just(&quot;[&quot;), just(&quot;]&quot;))\n     |&gt; map(Node)\n   alt(leaf, node)\n }\n \nSynonyms:\npears/combinators.lazy\npears/combinators lazy","url":"pears/combinators.html#lazy"},{"doc":"pears/combinators","title":"left","content":"pub fn left(\n  parser_1 p1: fn(Input(a)) -&gt;\n    Result(Parsed(a, b), ParseError(a)),\n  parser_2 p2: fn(Input(a)) -&gt;\n    Result(Parsed(a, c), ParseError(a)),\n) -&gt; fn(Input(a)) -&gt; Result(Parsed(a, b), ParseError(a))\n Applies the first parser and then the second parser, returning the result of the first parser.\nSynonyms:\npears/combinators.left\npears/combinators left","url":"pears/combinators.html#left"},{"doc":"pears/combinators","title":"many0","content":"pub fn many0(\n  parser: fn(Input(a)) -&gt; Result(Parsed(a, b), ParseError(a)),\n) -&gt; fn(Input(a)) -&gt; Result(Parsed(a, List(b)), ParseError(a))\n Applies the given parser zero or more times.\nSynonyms:\npears/combinators.many0\npears/combinators many0","url":"pears/combinators.html#many0"},{"doc":"pears/combinators","title":"many1","content":"pub fn many1(\n  parser: fn(Input(a)) -&gt; Result(Parsed(a, b), ParseError(a)),\n) -&gt; fn(Input(a)) -&gt; Result(Parsed(a, List(b)), ParseError(a))\n Applies the given parser one or more times.\nSynonyms:\npears/combinators.many1\npears/combinators many1","url":"pears/combinators.html#many1"},{"doc":"pears/combinators","title":"map","content":"pub fn map(\n  p: fn(Input(a)) -&gt; Result(Parsed(a, b), ParseError(a)),\n  fun: fn(b) -&gt; c,\n) -&gt; fn(Input(a)) -&gt; Result(Parsed(a, c), ParseError(a))\n Maps the result of a parser to a new value using a mapper function.\nSynonyms:\npears/combinators.map\npears/combinators map","url":"pears/combinators.html#map"},{"doc":"pears/combinators","title":"map_error","content":"pub fn map_error(\n  p: fn(Input(a)) -&gt; Result(Parsed(a, b), ParseError(a)),\n  fun: fn(ParseError(a)) -&gt; ParseError(a),\n) -&gt; fn(Input(a)) -&gt; Result(Parsed(a, b), ParseError(a))\n Maps the error of a parser to a new error using a mapper function.\nSynonyms:\npears/combinators.map_error\npears/combinators map_error","url":"pears/combinators.html#map_error"},{"doc":"pears/combinators","title":"maybe","content":"pub fn maybe(\n  parser: fn(Input(a)) -&gt; Result(Parsed(a, b), ParseError(a)),\n) -&gt; fn(Input(a)) -&gt; Result(Parsed(a, Option(b)), ParseError(a))\n Applies the given parser and wraps the result in an `Option`,\n returning `None` if the parser fails instead of an error.\nSynonyms:\npears/combinators.maybe\npears/combinators maybe","url":"pears/combinators.html#maybe"},{"doc":"pears/combinators","title":"none_of","content":"pub fn none_of(\n  items: List(a),\n) -&gt; fn(Input(a)) -&gt; Result(Parsed(a, a), ParseError(a))\n Consumes an item that is not included in the given list.\nSynonyms:\npears/combinators.none_of\npears/combinators none_of","url":"pears/combinators.html#none_of"},{"doc":"pears/combinators","title":"one_of","content":"pub fn one_of(\n  items: List(a),\n) -&gt; fn(Input(a)) -&gt; Result(Parsed(a, a), ParseError(a))\n Consumes an item that is included in the given list.\nSynonyms:\npears/combinators.one_of\npears/combinators one_of","url":"pears/combinators.html#one_of"},{"doc":"pears/combinators","title":"pair","content":"pub fn pair(\n  parser_1 p1: fn(Input(a)) -&gt;\n    Result(Parsed(a, b), ParseError(a)),\n  parser_2 p2: fn(Input(a)) -&gt;\n    Result(Parsed(a, c), ParseError(a)),\n) -&gt; fn(Input(a)) -&gt; Result(Parsed(a, #(b, c)), ParseError(a))\n\nSynonyms:\npears/combinators.pair\npears/combinators pair","url":"pears/combinators.html#pair"},{"doc":"pears/combinators","title":"recognize","content":"pub fn recognize(\n  parser: fn(Input(a)) -&gt; Result(Parsed(a, b), ParseError(a)),\n) -&gt; fn(Input(a)) -&gt; Result(Parsed(a, List(a)), ParseError(a))\n Creates a parser that returns the consumed tokens instead of it&#39;s parsed value.\nSynonyms:\npears/combinators.recognize\npears/combinators recognize","url":"pears/combinators.html#recognize"},{"doc":"pears/combinators","title":"right","content":"pub fn right(\n  parser_1 p1: fn(Input(a)) -&gt;\n    Result(Parsed(a, b), ParseError(a)),\n  parser_2 p2: fn(Input(a)) -&gt;\n    Result(Parsed(a, c), ParseError(a)),\n) -&gt; fn(Input(a)) -&gt; Result(Parsed(a, c), ParseError(a))\n Applies the first parser and then the second parser, returning the result of the second parser.\nSynonyms:\npears/combinators.right\npears/combinators right","url":"pears/combinators.html#right"},{"doc":"pears/combinators","title":"satisfying","content":"pub fn satisfying(\n  f: fn(a) -&gt; Bool,\n) -&gt; fn(Input(a)) -&gt; Result(Parsed(a, a), ParseError(a))\n Consumes a single item from the input that satisfies the given predicate function.\nSynonyms:\npears/combinators.satisfying\npears/combinators satisfying","url":"pears/combinators.html#satisfying"},{"doc":"pears/combinators","title":"sep_by0","content":"pub fn sep_by0(\n  parser: fn(Input(a)) -&gt; Result(Parsed(a, b), ParseError(a)),\n  separator: fn(Input(a)) -&gt; Result(Parsed(a, c), ParseError(a)),\n) -&gt; fn(Input(a)) -&gt; Result(Parsed(a, List(b)), ParseError(a))\n Parses zero or more occurrences of the given parser separated by the given separator.\nSynonyms:\npears/combinators.sep_by0\npears/combinators sep_by0","url":"pears/combinators.html#sep_by0"},{"doc":"pears/combinators","title":"sep_by1","content":"pub fn sep_by1(\n  parser: fn(Input(a)) -&gt; Result(Parsed(a, b), ParseError(a)),\n  separator: fn(Input(a)) -&gt; Result(Parsed(a, c), ParseError(a)),\n) -&gt; fn(Input(a)) -&gt; Result(Parsed(a, List(b)), ParseError(a))\n Parses one or more occurrences of the given parser separated by the given separator.\nSynonyms:\npears/combinators.sep_by1\npears/combinators sep_by1","url":"pears/combinators.html#sep_by1"},{"doc":"pears/combinators","title":"seq","content":"pub fn seq(\n  parsers: List(\n    fn(Input(a)) -&gt; Result(Parsed(a, b), ParseError(a)),\n  ),\n) -&gt; fn(Input(a)) -&gt; Result(Parsed(a, List(b)), ParseError(a))\n Applies the given parsers in sequence and returns a list of the results\nSynonyms:\npears/combinators.seq\npears/combinators seq","url":"pears/combinators.html#seq"},{"doc":"pears/combinators","title":"to","content":"pub fn to(\n  parser: fn(Input(a)) -&gt; Result(Parsed(a, b), ParseError(a)),\n  value: c,\n) -&gt; fn(Input(a)) -&gt; Result(Parsed(a, c), ParseError(a))\n Maps the result of a parser to a constant value.\nSynonyms:\npears/combinators.to\npears/combinators to","url":"pears/combinators.html#to"},{"doc":"pears/combinators","title":"unwrap","content":"pub fn unwrap(\n  parser: fn(Input(a)) -&gt;\n    Result(Parsed(a, Option(b)), ParseError(a)),\n  default: b,\n) -&gt; fn(Input(a)) -&gt; Result(Parsed(a, b), ParseError(a))\n Maps a parser that returns an `Option` to a parser that returns a value or a default value.\nSynonyms:\npears/combinators.unwrap\npears/combinators unwrap","url":"pears/combinators.html#unwrap"},{"doc":"pears/combinators","title":"pears/combinators","content":"","url":"pears/combinators.html"},{"doc":"pears/chars","title":"Char","content":"pub type Char =\n  String\n A grapheme is a user-perceived character.\n\nSynonyms:\npears/chars.Char\npears/chars Char","url":"pears/chars.html#Char"},{"doc":"pears/chars","title":"char","content":"pub fn char(\n  c: String,\n) -&gt; fn(Input(String)) -&gt;\n  Result(Parsed(String, String), ParseError(String))\n Parses a given character.\nSynonyms:\npears/chars.char\npears/chars char","url":"pears/chars.html#char"},{"doc":"pears/chars","title":"digit","content":"pub fn digit() -&gt; fn(Input(String)) -&gt;\n  Result(Parsed(String, String), ParseError(String))\n Parses a digit in the range 0-9.\nSynonyms:\npears/chars.digit\npears/chars digit","url":"pears/chars.html#digit"},{"doc":"pears/chars","title":"input","content":"pub fn input(s: String) -&gt; Input(String)\n Creates an `Input(Char)` from a string.\nSynonyms:\npears/chars.input\npears/chars input","url":"pears/chars.html#input"},{"doc":"pears/chars","title":"number","content":"pub fn number() -&gt; fn(Input(String)) -&gt;\n  Result(Parsed(String, Int), ParseError(String))\n Parses a number consisting of one or more digits.\nSynonyms:\npears/chars.number\npears/chars number","url":"pears/chars.html#number"},{"doc":"pears/chars","title":"string","content":"pub fn string(\n  str: String,\n) -&gt; fn(Input(String)) -&gt;\n  Result(Parsed(String, String), ParseError(String))\n Parses a given string.\nSynonyms:\npears/chars.string\npears/chars string","url":"pears/chars.html#string"},{"doc":"pears/chars","title":"whitespace","content":"pub fn whitespace() -&gt; fn(Input(String)) -&gt;\n  Result(Parsed(String, String), ParseError(String))\n Parses a character if is whitespace according to the Unicode standard.\nSynonyms:\npears/chars.whitespace\npears/chars whitespace","url":"pears/chars.html#whitespace"},{"doc":"pears/chars","title":"whitespace0","content":"pub fn whitespace0() -&gt; fn(Input(String)) -&gt;\n  Result(Parsed(String, List(String)), ParseError(String))\n Parses zero or more whitespace characters.\nSynonyms:\npears/chars.whitespace0\npears/chars whitespace0","url":"pears/chars.html#whitespace0"},{"doc":"pears/chars","title":"whitespace1","content":"pub fn whitespace1() -&gt; fn(Input(String)) -&gt;\n  Result(Parsed(String, List(String)), ParseError(String))\n Parses one or more whitespace characters.\nSynonyms:\npears/chars.whitespace1\npears/chars whitespace1","url":"pears/chars.html#whitespace1"},{"doc":"pears/chars","title":"pears/chars","content":"","url":"pears/chars.html"}]);